@@ -7,6 +7,9 @@ use English qw(-no_match_vars);
 use File::Spec;
 use Getopt::Long;
 use UNIVERSAL::require;
+use Data::Structure::Util qw/unbless/;
+
+require FusionInventory::Agent::Tools;
 
 my $default = {
     'additional-content'      => undef,
@@ -14,6 +17,7 @@ my $default = {
     'ca-cert-dir'             => undef,
     'ca-cert-file'            => undef,
     'color'                   => undef,
+    'conf-reload-interval'    => 0,
     'debug'                   => undef,
     'delaytime'               => 3600,
     'force'                   => undef,
@@ -38,14 +42,11 @@ my $default = {
     'scan-profiles'           => undef,
     'server'                  => undef,
     'tag'                     => undef,
+    'tasks'                   => undef,
     'timeout'                 => 180,
     'user'                    => undef,
     # deprecated options
     'stdout'                  => undef,
-    # multi-values options that will be converted to array ref
-    'httpd-trust'             => "",
-    'no-task'                 => "",
-    'no-category'             => ""
 };
 
 my $deprecated = {
@@ -55,17 +56,46 @@ my $deprecated = {
     },
 };
 
+my $confReloadIntervalMinValue = 60;
+
 sub new {
     my ($class, %params) = @_;
 
     my $self = {};
     bless $self, $class;
     $self->_loadDefaults();
+
+    $self->_loadFromBackend($params{options}->{'conf-file'}, $params{options}->{config}, $params{confdir});
+
+    $self->_loadUserParams($params{options});
+
+    $self->_checkContent();
+
+    if (defined($params{options}->{'conf-file'})) {
+        $self->{'conf-file'} = $params{options}->{'conf-file'}
+    }
+
+    return $self;
+}
+
+sub reloadFromInputAndBackend {
+    my ($self, $confDir) = @_;
+
+    $self->_loadDefaults;
+
+    $self->_loadFromBackend($self->{'conf-file'}, $self->{config}, $confDir);
+
+    $self->_checkContent();
+}
+
+sub _loadFromBackend {
+    my ($self, $confFile, $config, $confdir) = @_;
+
     my $backend =
-        $params{options}->{'conf-file'} ? 'file'                     :
-        $params{options}->{config}      ? $params{options}->{config} :
-        $OSNAME eq 'MSWin32'            ? 'registry'                 :
-                                          'file';
+        $confFile            ? 'file'      :
+        $config              ? $config     :
+        $OSNAME eq 'MSWin32' ? 'registry'  :
+                               'file';
 
     SWITCH: {
         if ($backend eq 'registry') {
@@ -77,8 +107,8 @@ sub new {
 
         if ($backend eq 'file') {
             $self->_loadFromFile({
-                file      => $params{options}->{'conf-file'},
-                directory => $params{confdir},
+                file      => $confFile,
+                directory => $confdir,
             });
             last SWITCH;
         }
@@ -89,12 +119,6 @@ sub new {
 
         die "Unknown configuration backend '$backend'\n";
     }
-
-    $self->_loadUserParams($params{options});
-
-    $self->_checkContent();
-
-    return $self;
 }
 
 sub _loadDefaults {
@@ -253,9 +277,12 @@ sub _checkContent {
             httpd-trust
             no-task
             no-category
+            tasks
             /) {
 
-        if ($self->{$option}) {
+        # Check if defined AND SCALAR
+        # to avoid split a ARRAY ref or HASH ref...
+        if ($self->{$option} && ref($self->{$option}) eq '') {
             $self->{$option} = [split(/,/, $self->{$option})];
         } else {
             $self->{$option} = [];
@@ -269,6 +296,24 @@ sub _checkContent {
         File::Spec->rel2abs($self->{'ca-cert-dir'}) if $self->{'ca-cert-dir'};
     $self->{'logfile'} =
         File::Spec->rel2abs($self->{'logfile'}) if $self->{'logfile'};
+
+    # conf-reload-interval option
+    # If value is less than the required minimum, we force it to that
+    # minimum because it's useless to reload the config so often and,
+    # furthermore, it can cause a loss of performance
+    if ($self->{'conf-reload-interval'} != 0) {
+        if ($self->{'conf-reload-interval'} < 0) {
+            $self->{'conf-reload-interval'} = 0;
+        } elsif ($self->{'conf-reload-interval'} < $confReloadIntervalMinValue) {
+            $self->{'conf-reload-interval'} = $confReloadIntervalMinValue;
+        }
+    }
+}
+
+sub isParamArrayAndFilled {
+    my ($self, $paramName) = @_;
+
+    return FusionInventory::Agent::Tools::isParamArrayAndFilled($self, $paramName);
 }
 
 1;
