@@ -171,7 +171,15 @@ sub _handle_deploy {
     return unless $sha512 =~ /^(.)(.)(.{6})/;
     my $subFilePath = $1.'/'.$2.'/'.$3;
 
+    my $logger = $self->{logger};
+
     Digest::SHA->require();
+    if ($EVAL_ERROR) {
+        $logger->error("Failed to load Digest::SHA: $EVAL_ERROR");
+        # Return 501 (Not Implemented) to client with reason
+        $client->send_error(501, 'Digest::SHA perl library is missing');
+        return 501;
+    }
 
     my $path;
     LOOP: foreach my $target ($self->{agent}->getTargets()) {
@@ -288,7 +296,8 @@ sub init {
         LocalAddr => $self->{ip},
         LocalPort => $self->{port},
         Reuse     => 1,
-        Timeout   => 5
+        Timeout   => 5,
+        Blocking  => 0
     );
 
     if (!$self->{listener}) {
@@ -299,6 +308,8 @@ sub init {
     $logger->debug(
         $log_prefix . "HTTPD service started on port $self->{port}"
     );
+
+    return 1;
 }
 
 sub handleRequests {
@@ -313,6 +324,8 @@ sub handleRequests {
     my $clientIp = inet_ntoa($iaddr);
     my $request = $client->get_request();
     $self->_handle($client, $request, $clientIp);
+
+    return 1;
 }
 
 1;
