@@ -28,7 +28,9 @@ my %fields = (
                              SSH_KEY ARCH BOOT_TIME/ ],
     ACCESSLOG        => [ qw/USERID LOGDATE/ ],
 
-    ANTIVIRUS        => [ qw/COMPANY ENABLED GUID NAME UPTODATE VERSION/ ],
+    ANTIVIRUS        => [ qw/COMPANY ENABLED GUID NAME UPTODATE VERSION
+                             DATFILECREATION DATFILEVERSION ENGINEVERSION32
+                             ENGINEVERSION64/ ],
     BATTERIES        => [ qw/CAPACITY CHEMISTRY DATE NAME SERIAL MANUFACTURER
                              VOLTAGE/ ],
     CONTROLLERS      => [ qw/CAPTION DRIVER NAME MANUFACTURER PCICLASS VENDORID
@@ -68,6 +70,7 @@ my %fields = (
     PROCESSES        => [ qw/USER PID CPUUSAGE MEM VIRTUALMEMORY TTY STARTED
                              CMD/ ],
     REGISTRY         => [ qw/NAME REGVALUE HIVE/ ],
+    REMOTE_MGMT      => [ qw/ID TYPE/ ],
     RUDDER           => [ qw/AGENT UUID HOSTNAME/ ],
     SLOTS            => [ qw/DESCRIPTION DESIGNATION NAME STATUS/ ],
     SOFTWARES        => [ qw/COMMENTS FILESIZE FOLDER FROM HELPLINK INSTALLDATE
@@ -173,6 +176,10 @@ sub mergeContent {
                     $self->setHardware($content->{$section});
                     last SWITCH;
                 }
+                if ($section eq 'OPERATINGSYSTEM') {
+                    $self->setOperatingSystem($content->{$section});
+                    last SWITCH;
+                }
                 if ($section eq 'BIOS') {
                     $self->setBios($content->{$section});
                     last SWITCH;
