@@ -50,7 +50,24 @@ sub new {
     }
 
     ($self->{session}, my $error) = Net::SNMP->session(%options);
-    die $error unless $self->{session};
+    if (!$self->{session}) {
+        die "no response from host $params{hostname}\n"
+            if $error =~ /^No response from remote host/;
+        die "authentication error on host $params{hostname}\n"
+            if $error =~ /^Received usmStats(WrongDigests|UnknownUserNames)/;
+        die $error . "\n";
+    }
+
+    if ($version ne 'snmpv3') {
+        my $oid = '.1.3.6.1.2.1.1.1.0';
+        my $response = $self->{session}->get_request(
+            -varbindlist => [$oid]
+        );
+        die "no response from host $params{hostname}\n"
+            if !$response;
+        die "no response from host $params{hostname}\n"
+            if $response->{$oid} =~ /No response from remote host/;
+    }
 
     bless $self, $class;
 
@@ -111,11 +128,10 @@ sub get {
     return unless $oid;
 
     my $session = $self->{session};
+    my %options = (-varbindlist => [$oid]);
+    $options{'-contextname'} = $self->{context} if $self->{context};
 
-    my $response = $session->get_request(
-        -varbindlist => [$oid],
-        ($self->{context} ? (-contextname => $self->{context}) : ())
-    );
+    my $response = $session->get_request(%options);
 
     return unless $response;
 
@@ -135,12 +151,11 @@ sub walk {
     return unless $oid;
 
     my $session = $self->{session};
+    my %options = (-baseoid => $oid);
+    $options{'-contextname'}    = $self->{context} if $self->{context};
+    $options{'-maxrepetitions'} = 1                if $session->version() != 0;
 
-    my $response = $session->get_table(
-        -baseoid => $oid,
-        -maxrepetitions => 1,
-        ($self->{context} ? (-contextname => $self->{context}) : ())
-    );
+    my $response = $session->get_table(%options);
 
     return unless $response;
 
