@@ -67,20 +67,9 @@ sub _getInterfaces {
             $interface->{IPMASK}
         );
 
-        if ($interface->{DESCRIPTION} =~ /^(\S+)(\d+)/) {
-            my $nic = $1;
-            my $num = $2;
-
-            $interface->{SPEED} =
-                $nic =~ /aggr/   ? undef                       :
-                $nic =~ /dmfe/   ? undef                       :
-                $nic =~ /bge/    ? _check_bge_nic($nic, $num)  :
-                $nic =~ /nxge/   ? _check_nxge_nic($nic, $num) :
-                $nic =~ /ce/     ? _check_ce_nic($nic, $num)   :
-                $nic =~ /ipge/   ? _check_ce_nic($nic, $num)   :
-                $nic =~ /e1000g/ ? _check_ce_nic($nic, $num)   :
-                                   _check_nic($nic, $num);
-        }
+        $interface->{SPEED} = _getInterfaceSpeed(
+            name => $interface->{DESCRIPTION}
+        );
     }
 
     my $zone = getZone();
@@ -101,127 +90,30 @@ sub _getInterfaces {
     return @interfaces;
 }
 
+sub  _getInterfaceSpeed {
+    my (%params) = @_;
 
-# Function to test Quad Fast-Ethernet, Fast-Ethernet, and
-# Gigabit-Ethernet (i.e. qfe_, hme_, ge_, fjgi_)
-sub _check_nic {
-    my ($nic, $num) = @_;
+    my $command;
 
-    my $speed = getFirstMatch(
-        command => "/usr/sbin/ndd -get /dev/$nic link_speed",
-        pattern => qr/^(\d+)/
-    );
+    if ($params{name}) {
+        return unless $params{name} =~ /^(\S+)(\d+)/;
+        my $type     = $1;
+        my $instance = $2;
 
-    my $duplex = getFirstMatch(
-        command => "/usr/sbin/ndd -get /dev/$nic link_mode",
-        pattern => qr/^(\d+)/
-    );
+        return if $type eq 'aggr';
+        return if $type eq 'dmfe';
 
-    my $arg = $nic =~ /ge/ ? 'adv_1000autoneg_cap' : 'adv_autoneg_cap';
-    my $auto = getFirstMatch(
-        command => "/usr/sbin/ndd -get /dev/$nic $arg",
-        pattern => qr/^(\d+)/
-    );
-
-    return _get_link_info($speed, $duplex, $auto);
-}
-
-# Function to test a Gigabit-Ethernet (i.e. ce_).
-# Function to test a Intel 82571-based ethernet controller port (i.e. ipge_).
-sub _check_ce_nic {
-    my ($nic, $num) = @_;
-
-    my $speed = getFirstMatch(
-        command => "/usr/bin/kstat -m $nic -i $num -s link_speed",
-        pattern => qr/^\s*link_speed+\s*(\d+)/
-    );
-
-    my $duplex = getFirstMatch(
-        command => "/usr/bin/kstat -m $nic -i $num -s link_duplex",
-        pattern => qr/^\s*link_duplex+\s*(\d+)/
-    );
-
-    my $auto = getFirstMatch(
-        command => "/usr/bin/kstat -m $nic -i $num -s cap_autoneg",
-        pattern => qr/^\s*cap_autoneg+\s*(\d+)/
-    );
-
-    return _get_link_info($speed, $duplex, $auto);
-
-}
-
-# Function to test Sun BGE interface on Sun Fire V210 and V240.
-# The BGE is a Broadcom BCM5704 chipset. There are four interfaces
-# on the V210 and V240. (i.e. bge_)
-sub _check_bge_nic {
-    my ($nic, $num) = @_;
-
-    my $speed = getFirstMatch(
-        command => "/usr/sbin/ndd -get /dev/$nic$num link_speed",
-        pattern => qr/^(\d+)/
-    );
-
-    my $duplex = getFirstMatch(
-        command => "/usr/sbin/ndd -get /dev/$nic$num link_duplex",
-        pattern => qr/^(\d+)/
-    );
-
-    my $auto = getFirstMatch(
-        command => "/usr/sbin/ndd -get /dev/$nic$num adv_autoneg_cap",
-        pattern => qr/^(\d+)/
-    );
-
-    return _get_link_info($speed, $duplex, $auto);
-}
-
-
-# Function to test Sun NXGE interface on Sun Fire Tx000.
-sub _check_nxge_nic {
-    my ($nic, $num) = @_;
-
-    #nxge0           link: up        speed: 1000  Mbps       duplex: full
-    my ($speed, $unit, $duplex) = getFirstMatch(
-        command => "/usr/sbin/dladm show-dev $nic$num",
-        pattern => qr/
-            $nic$num \s+
-            link:   \s \S+   \s+
-            speed:  \s (\d+) \s+ (\S+) \s+
-            duplex: \s (\S+)
-        /x
-    );
-    return $speed . ' ' . $unit . ' ' . $duplex;
-}
-
-sub _get_link_info {
-    my ($speed, $duplex, $auto) = @_;
-
-    my $info;
-
-    if ($speed) {
-        $info =
-            $speed == 0    ? "10 Mb/s"  :
-            $speed == 10   ? "10 Mb/s"  :
-            $speed == 100  ? "100 Mb/s" :
-            $speed == 1000 ? "1 Gb/s"   :
-                             "ERROR"    ;
+        $command = "/usr/bin/kstat -m $type -i $instance -s link_speed";
     }
 
-    if ($duplex) {
-        $info .=
-            $duplex == 2 ? " FDX"     :
-            $duplex == 1 ? " HDX"     :
-            $duplex == 0 ? " UNKNOWN" :
-                           " ERROR"   ;
-    }
+    my $speed = getFirstMatch(
+        %params,
+        command => $command,
+        pattern => qr/^\s*link_speed+\s*(\d+)/,
+    );
 
-    if ($auto) {
-        $info .=
-            $auto == 0 ? " AUTOSPEED ON"    :
-            $auto == 1 ? " AUTOSPEED OFF"   :
-                         " ERROR"           ;
-    }
-
-    return $info;
+    # By default, kstat reports speed as Mb/s, no need to normalize
+    return $speed;
 }
 
 sub _parseIfconfig {
@@ -291,7 +183,7 @@ sub _parseDladm {
             $line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
         $interface->{DESCRIPTION} = $1;
         $interface->{MACADDR}     = $2;
-        $interface->{SPEED}       = $3 . " " . $4 . " " . $5;
+        $interface->{SPEED}       = getCanonicalInterfaceSpeed($3 . $4);
         $interface->{STATUS}      = 'Up' if $line =~ /UP/;
         push @interfaces, $interface;
     }
@@ -312,7 +204,7 @@ sub _parsefcinfo {
             if $line =~ /HBA Port WWN:\s+(\S+)/;
         $interface->{DESCRIPTION} .= " " . $1
             if $line =~ /OS Device Name:\s+(\S+)/;
-        $interface->{SPEED} = $1
+        $interface->{SPEED} = getCanonicalInterfaceSpeed($1)
             if $line =~ /Current Speed:\s+(\S+)/;
         $interface->{WWN} = $1
             if $line =~ /Node WWN:\s+(\S+)/;
@@ -328,6 +220,7 @@ sub _parsefcinfo {
             if $line =~ /online/;
 
         if ($interface->{DESCRIPTION} && $interface->{WWN}) {
+            $interface->{TYPE}   = 'fibrechannel';
             $interface->{STATUS} = 'Down' if !$interface->{STATUS};
 
             push @interfaces, $interface;
