@@ -33,14 +33,14 @@ sub doInventory {
             Access => KEY_READ | KEY_WOW64_64 ## no critic (ProhibitBitwise)
         }) or $logger->error("Can't open HKEY_LOCAL_MACHINE key: $EXTENDED_OS_ERROR");
         my $officeKey64 = $machKey64->{"SOFTWARE/Microsoft/Office"};
-        push @licenses, _scanOffice($officeKey64 ) if $officeKey64;
+        push @licenses, _scanOfficeLicences($officeKey64 ) if $officeKey64;
 
         my $machKey32 = $Registry->Open('LMachine', {
             Access => KEY_READ | KEY_WOW64_32 ## no critic (ProhibitBitwise)
         }) or $logger->error("Can't open HKEY_LOCAL_MACHINE key: $EXTENDED_OS_ERROR");
 
         my $officeKey32 = $machKey32->{"SOFTWARE/Microsoft/Office"};
-        push @licenses, _scanOffice($officeKey32) if $officeKey32;
+        push @licenses, _scanOfficeLicences($officeKey32) if $officeKey32;
     } else {
         my $machKey = $Registry->Open('LMachine', {
             Access => KEY_READ ## no critic (ProhibitBitwise)
@@ -49,59 +49,72 @@ sub doInventory {
         );
 
         my $officeKey = $machKey->{"SOFTWARE/Microsoft/Office"};
-        push @licenses, _scanOffice($officeKey) if $officeKey;
+        push @licenses, _scanOfficeLicences($officeKey) if $officeKey;
     }
 
     foreach my $license (@licenses) {
-        $params{inventory}->addEntry(
+        $inventory->addEntry(
             section => 'LICENSEINFOS',
             entry   => $license
         );
     }
 }
 
-sub _scanOffice {
+sub _scanOfficeLicences {
     my ($key) = @_;
 
-    my %license = (
-        PRODUCTID => $key->{ProductID},
-        UPDATE    => $key->{SPLevel},
-        OEM       => $key->{OEM},
-        FULLNAME  => encodeFromRegistry($key->{ProductName}) ||
-                     encodeFromRegistry($key->{ConvertToEdition}),
-        NAME      => encodeFromRegistry($key->{ProductNameNonQualified}) ||
-                     encodeFromRegistry($key->{ProductNameVersion})
-    );
+    # registry data structure:
+    # SOFTWARE/Microsoft/Office
+    # └── x.y
+    #     └── Registration
+    #         └── UUID
+    #             └── DigitalProductID:value
+    #             └── ProductID:value
+    #             └── ...
 
-    if ($key->{DigitalProductID}) {
-        $license{KEY} = decodeMicrosoftKey($key->{DigitalProductID});
+    my @licences;
+
+    foreach my $versionKey (keys %{$key}) {
+        my $registrationKey = $key->{$versionKey}->{'Registration/'};
+        next unless $registrationKey;
+
+        foreach my $uuidKey (keys %{$registrationKey}) {
+            next unless $registrationKey->{$uuidKey}->{'/DigitalProductID'};
+            push @licences, _getOfficeLicense($registrationKey->{$uuidKey});
+        }
     }
 
-    if ($key->{TrialType} && $key->{TrialType} =~ /(\d+)$/) {
-        $license{TRIAL} = int($1);
+    return @licences;
+}
+
+sub _getOfficeLicense {
+    my ($key) = @_;
+
+    my $license = {
+        KEY       => decodeMicrosoftKey($key->{'/DigitalProductID'}),
+        PRODUCTID => $key->{'/ProductID'},
+        UPDATE    => $key->{'/SPLevel'},
+        OEM       => $key->{'/OEM'},
+        FULLNAME  => encodeFromRegistry($key->{'/ProductName'}) ||
+                     encodeFromRegistry($key->{'/ConvertToEdition'}),
+        NAME      => encodeFromRegistry($key->{'/ProductNameNonQualified'}) ||
+                     encodeFromRegistry($key->{'/ProductNameVersion'})
+    };
+
+    if ($key->{'/TrialType'} && $key->{'/TrialType'} =~ /(\d+)$/) {
+        $license->{TRIAL} = int($1);
     }
 
     my @products;
-    foreach my $entry (keys %$key) {
-        next unless $entry =~ s/\/(\w+)NameVersion$//;
-        my $product = $1;
-        next unless $key->{$product."NameVersion"};
-        push @products, $product;
+    foreach my $variable (keys %$key) {
+        next unless $variable =~ m/\/(\w+)NameVersion$/;
+        push @products, $1;
     }
     if (@products) {
-        $license{COMPONENTS} = join('/', @products);
+        $license->{COMPONENTS} = join('/', sort @products);
     }
 
-    my @licenses;
-    push @licenses, \%license if $license{KEY};
-
-    foreach my $subKey (keys %$key) {
-        # skip variables
-        next if $subKey =~ m{^/};
-        push @licenses, _scanOffice($key->{$subKey});
-    }
-
-    return @licenses;
+    return $license;
 }
 
 1;
