@@ -92,11 +92,21 @@ sub _getDrives {
             SCSI_UNID    => $object->{SCSITargetId},
         };
 
+        # Cleanup field which may contain spaces
+        $drive->{FIRMWARE} =~ s/\s+$//
+            if $drive->{FIRMWARE};
+
         $drive->{DISKSIZE} = int($object->{Size} / (1024 * 1024))
             if $object->{Size};
 
-        $drive->{SERIAL} = $object->{SerialNumber}
-            if $object->{SerialNumber} && $object->{SerialNumber} !~ /^ +$/;
+        if ($object->{SerialNumber} && $object->{SerialNumber} !~ /^ +$/) {
+            # Try to decode serial only for known case
+            if ($drive->{MODEL} =~ /VBOX HARDDISK ATA/) {
+                $drive->{SERIAL} = _decodeSerialNumber($object->{SerialNumber});
+            } else {
+                $drive->{SERIAL} = $object->{SerialNumber};
+            }
+        }
 
         push @drives, $drive;
     }
@@ -104,4 +114,29 @@ sub _getDrives {
     return @drives;
 }
 
+sub _decodeSerialNumber {
+    my ($serial) = @_ ;
+
+    return $serial unless ($serial =~ /^[0-9a-f]+$/);
+
+    # serial is a space padded string encoded in hex words (4 hex-digits by word)
+    return $serial if length($serial) % 4 ;
+
+    # Map hex-encoded string to list of chars
+    my @chars = map { chr hex } unpack("(a[2])*", $serial);
+
+    $serial = '';
+
+    # Re-order chars
+    while (@chars) {
+        my $next = shift(@chars);
+        $serial .= shift(@chars) . $next ;
+    }
+
+    # Strip trailing spaces
+    $serial =~ s/ *$//;
+
+    return $serial;
+}
+
 1;
