@@ -4,8 +4,14 @@ use strict;
 use warnings;
 use base 'Exporter';
 
+# Constant for ethtool system call
+use constant SIOCETHTOOL   =>     0x8946 ; # See linux/sockios.h
+use constant ETHTOOL_GSET  => 0x00000001 ; # See linux/ethtool.h
+use constant SPEED_UNKNOWN =>      65535 ; # See linux/ethtool.h, to be read as -1
+
 use English qw(-no_match_vars);
 use Memoize;
+use Socket qw(PF_INET SOCK_DGRAM);
 
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Unix;
@@ -19,6 +25,7 @@ our @EXPORT = qw(
     getInfoFromSmartctl
     getInterfacesFromIfconfig
     getInterfacesFromIp
+    getInterfacesInfosFromIoctl
 );
 
 memoize('getDevicesFromUdev');
@@ -385,6 +392,47 @@ sub getInterfacesFromIfconfig {
 
     return @interfaces;
 }
+
+sub getInterfacesInfosFromIoctl {
+    my (%params) = (
+        interface => 'eth0',
+        @_
+    );
+
+    return unless $params{interface};
+
+    my $logger = $params{logger};
+
+    socket(my $socket, PF_INET, SOCK_DGRAM, 0)
+        or return ;
+
+    # Pack command in ethtool_cmd struct
+    my $cmd = pack("L3SC6L2SC2L3", ETHTOOL_GSET);
+
+    # Pack request for ioctl
+    my $request = pack("a16p", $params{interface}, $cmd);
+
+    my $retval = ioctl($socket, SIOCETHTOOL, $request) || -1;
+    return if ($retval < 0);
+
+    # Unpack returned datas
+    my @datas = unpack("L3SC6L2SC2L3", $cmd);
+
+    # Actually only speed value is requested and extracted
+    my $datas = {
+        SPEED => $datas[3]|$datas[12]<<16
+    };
+
+    # Forget speed value if got unknown speed special value
+    if ($datas->{SPEED} == SPEED_UNKNOWN) {
+        delete $datas->{SPEED};
+        $params{logger}->debug2("Unknown speed found on $params{interface}")
+            if $params{logger};
+    }
+
+    return $datas;
+}
+
 sub getInterfacesFromIp {
     my (%params) = (
         command => '/sbin/ip addr show',
@@ -563,6 +611,20 @@ Availables parameters:
 
 =back
 
+=head2 getInterfacesInfosFromIoctl(%params)
+
+Returns interface datas, by parsing results from ethtool system call request.
+
+Availables parameters:
+
+=over
+
+=item logger a logger object
+
+=item interface the interface name to use (default: eth0)
+
+=back
+
 =head2 getInterfacesFromIp(%params)
 
 Returns the list of interfaces, by parsing ip command output.
